@@ -147,31 +147,6 @@ static server_rec *g_pServer             = NULL;
 **  _________________________________________________________________
 */
 
-/* 
- *  the main Apache API config structure 
- */
-module MODULE_VAR_EXPORT ssl_module = {
-    STANDARD_MODULE_STUFF,
-    init_Module,              /* module initializer                  */
-    config_perdir_create,     /* create per-dir    config structures */
-    config_perdir_merge,      /* merge  per-dir    config structures */
-    config_server_create,     /* create per-server config structures */
-    config_server_merge,      /* merge  per-server config structures */
-    ssl_cmds,                 /* table of config file commands       */
-    NULL,                     /* [#8] MIME-typed-dispatched handlers */
-    NULL,                     /* [#1] URI to filename translation    */
-    ssl_hook_Auth,            /* [#4] validate user id from request  */
-    NULL,                     /* [#5] check if the user is ok _here_ */
-    ssl_hook_Access,          /* [#2] check access by host address   */
-    NULL,                     /* [#6] determine MIME type            */
-    ssl_hook_Fixup,           /* [#7] pre-run fixups                 */
-    NULL,                     /* [#9] log a transaction              */
-    NULL,                     /* [#3] header parser                  */
-    NULL,                     /* child_init                          */ 
-    NULL,                     /* child_exit                          */ 
-    NULL                      /* [#0] post read-request              */ 
-};
-
 /* 
  *  the table of configuration directives we provide 
  */
@@ -231,6 +206,31 @@ static command_rec ssl_cmds[] = {
     { NULL },
 };
 
+/* 
+ *  the main Apache API config structure 
+ */
+module MODULE_VAR_EXPORT ssl_module = {
+    STANDARD_MODULE_STUFF,
+    init_Module,              /* module initializer                  */
+    config_perdir_create,     /* create per-dir    config structures */
+    config_perdir_merge,      /* merge  per-dir    config structures */
+    config_server_create,     /* create per-server config structures */
+    config_server_merge,      /* merge  per-server config structures */
+    ssl_cmds,                 /* table of config file commands       */
+    NULL,                     /* [#8] MIME-typed-dispatched handlers */
+    NULL,                     /* [#1] URI to filename translation    */
+    ssl_hook_Auth,            /* [#4] validate user id from request  */
+    NULL,                     /* [#5] check if the user is ok _here_ */
+    ssl_hook_Access,          /* [#2] check access by host address   */
+    NULL,                     /* [#6] determine MIME type            */
+    ssl_hook_Fixup,           /* [#7] pre-run fixups                 */
+    NULL,                     /* [#9] log a transaction              */
+    NULL,                     /* [#3] header parser                  */
+    NULL,                     /* child_init                          */ 
+    NULL,                     /* child_exit                          */ 
+    NULL                      /* [#0] post read-request              */ 
+};
+
 
 /*  _________________________________________________________________
 **
@@ -311,9 +311,9 @@ static void *config_perdir_merge(pool *p, void *basev, void *addv)
     SSLDirConfigRec *new  = (SSLDirConfigRec *)ap_palloc(p, 
                                                sizeof(SSLDirConfigRec));
 
-	cfgMergeTable(tbRequiredCiphers);
-	cfgMergeTable(tbBannedCiphers);
-	cfgMerge(bSSLRequired, FALSE);
+    cfgMergeTable(tbRequiredCiphers);
+    cfgMergeTable(tbBannedCiphers);
+    cfgMerge(bSSLRequired, FALSE);
 
     return new;
 }
@@ -492,7 +492,7 @@ static void init_Module(server_rec *s, pool *p)
     /* 
      *  Allegedly this is only run once when included as a DSO (though the
      *  value of doing that is questionable, since the rest of the code
-     *  doesn't work without it).
+     *  doesn't work without this module).
      */
     g_bFirstTime = FALSE;
 #endif
@@ -529,13 +529,13 @@ static void init_Module(server_rec *s, pool *p)
      *  make sure the required SSL session cache is configured
      */
     if (!g_szCacheServerPort) {
-        ap_log_error(APLOG_MARK, APLOG_ERR, s,
+        ap_log_error(APLOG_MARK, APLOG_ERR|APLOG_NOERRNO, s,
                      "mod_ssl: Required SSLCacheServerPort missing");
         ap_clear_pool(p);
         exit(1);
     }
     if (!g_szCacheServerPath) {
-        ap_log_error(APLOG_MARK, APLOG_ERR, s,
+        ap_log_error(APLOG_MARK, APLOG_ERR|APLOG_NOERRNO, s,
                      "mod_ssl: Required SSLCacheServerPath missing");
         ap_clear_pool(p);
         exit(1);
@@ -875,8 +875,8 @@ static int init_VerifyFlags(SSLConfigRec *pConfig)
     fb->ssl = __SSLeay SSL_new(pConfig->pSSLCtx);
     __SSLeay SSL_set_fd(fb->ssl, fb->fd);
 #if FREE_SESSION
-    register_cleanup(conn->pool, fb->ssl, 
-                     SSL_HOOK_CloseConnection, SSL_HOOK_CloseConnection);
+    ap_register_cleanup(conn->pool, conn, 
+                        SSL_HOOK_CloseConnection, SSL_HOOK_CloseConnection);
 #endif
 
     /*
@@ -906,8 +906,7 @@ static int init_VerifyFlags(SSLConfigRec *pConfig)
         if (pConfig->nVerifyClient != VERIFY_NONE) {
             char *s;
             X509 *xs;
-            xs = __SSLeay SSL_get_peer_certificate(fb->ssl);
-            if (xs) {
+            if ((xs = __SSLeay SSL_get_peer_certificate(fb->ssl)) != NULL) {
                 s = __SSLeay X509_NAME_oneline(X509_get_subject_name(xs), NULL, 0);
                 conn->client->szClientX509 = ap_pstrdup(conn->pool, s);
                 free(s);
@@ -945,10 +944,14 @@ static int init_VerifyFlags(SSLConfigRec *pConfig)
  * Remove from SSL_free: if (s->ctx) SSL_CTX_free(s->ctx);
  * But there's more, which I haven't found. -- ben
  */
-static void SSL_HOOK_CloseConnection(void *_ssl)
+static void SSL_HOOK_CloseConnection(void *_conn)
 {
-    SSL *ssl = _ssl;
-    __SSLeay SSL_free(_ssl);
+    conn_rec *conn = _conn;
+    SSL *ssl = conn->client->ssl;
+
+    __SSLeay SSL_set_shutdown(ssl, SSL_SENT_SHUTDOWN|SSL_RECEIVED_SHUTDOWN);
+    __SSLeay SSL_free(ssl);
+    /* ap_log_error(APLOG_MARK,APLOG_DEBUG|APLOG_NOERRNO,conn->server, "Closed"); */
 }
 #endif
 
@@ -1435,6 +1438,7 @@ static void ssl_log_own(SSLConfigRec *pConfig, const char *msg, ...)
     fprintf(pConfig->fileLogFile, str);
     fflush(pConfig->fileLogFile);
 
+    va_end(ap);
     return;
 }
 
